/* eslint-disable no-magic-numbers */
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { InitialStoreState } from '@/utils/testing/getReduxStoreActionsListener';
import { StoreType } from '@/redux/store';
import { render, screen } from '@testing-library/react';
import { bioEntitiesContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { BIOENTITY_INITIAL_STATE_MOCK } from '@/redux/bioEntity/bioEntity.mock';
import { ConnectedBioEntitiesList } from './ConnectedBioEntitiesList.component';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <ConnectedBioEntitiesList />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('ConnectedBioEntitiesList', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  it('renders loading indicator when bioEntityLoading is pending', () => {
    renderComponent({
      bioEntity: {
        ...BIOENTITY_INITIAL_STATE_MOCK,
        loading: 'pending',
      },
    });

    const loadingIndicator = screen.getByTestId('loading-indicator');

    expect(loadingIndicator).toBeVisible();
  });

  it('renders list of bio entities when bioEntityData is available', () => {
    const bioEntityData = [bioEntitiesContentFixture[0]];

    renderComponent({
      bioEntity: {
        ...BIOENTITY_INITIAL_STATE_MOCK,
        data: [
          {
            searchQueryElement: '',
            loading: 'succeeded',
            error: { name: '', message: '' },
            data: bioEntityData,
          },
        ],
      },
    });

    expect(screen.queryByTestId('loading-indicator')).not.toBeInTheDocument();
    expect(screen.queryByText(bioEntitiesContentFixture[0].bioEntity.name)).toBeVisible();
  });
});
