import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import {
  numberOfChemicalsSelector,
  loadingChemicalsStatusSelector,
} from '@/redux/chemicals/chemicals.selectors';
import { displayChemicalsList } from '@/redux/drawer/drawer.slice';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { AccordionItem, AccordionItemHeading, AccordionItemButton } from '@/shared/Accordion';

export const ChemicalsAccordion = (): JSX.Element => {
  const dispatch = useAppDispatch();
  const chemicalsNumber = useAppSelector(numberOfChemicalsSelector);
  const chemicalsState = useAppSelector(loadingChemicalsStatusSelector);

  const isPending = chemicalsState === 'pending';
  const isSucceeded = chemicalsState === 'succeeded';
  const isChemicalsEmpty = chemicalsNumber === SIZE_OF_EMPTY_ARRAY;

  const onAccordionClick = (): void => {
    dispatch(displayChemicalsList());
  };

  return (
    <AccordionItem>
      <AccordionItemHeading>
        <AccordionItemButton
          variant="non-expandable"
          onClick={onAccordionClick}
          disabled={isPending || isChemicalsEmpty}
        >
          Small molecule targets
          {isPending && ' (Loading...)'}
          {isSucceeded && ` (${chemicalsNumber})`}
        </AccordionItemButton>
      </AccordionItemHeading>
    </AccordionItem>
  );
};
