import { getMultiBioEntity } from '@/redux/bioEntity/bioEntity.thunks';
import { closeDrawer, openBioEntityDrawerById, selectTab } from '@/redux/drawer/drawer.slice';
import { AppDispatch } from '@/redux/store';
import { searchFitBounds } from '@/services/pluginsManager/map/triggerSearch/searchFitBounds';
import { ElementSearchResult } from '@/types/models';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { clearBioEntitiesData } from '@/redux/bioEntity/bioEntity.slice';
import { Point } from '@/types/map';
import { findClosestBioEntityPoint } from './findClosestBioEntityPoint';

type SearchConfig = {
  point: Point;
  searchDistance?: string;
  maxZoom: number;
  zoom: number;
  hasFitBounds?: boolean;
  isResultDrawerOpen?: boolean;
};
/* prettier-ignore */
export const handleAliasResults =
  (dispatch: AppDispatch, closestSearchResult: ElementSearchResult, { hasFitBounds, maxZoom, point, searchDistance, zoom, isResultDrawerOpen }: SearchConfig) =>
    async ({ id }: ElementSearchResult): Promise<void> => {
      const bioEntityContents = await dispatch(
        getMultiBioEntity({
          searchQueries: [id.toString()],
          isPerfectMatch: true
        }),
      ).unwrap();

      if (searchDistance) {

        const matchingBioEntityFound = findClosestBioEntityPoint(bioEntityContents, searchDistance, maxZoom, zoom, point);

        if (!matchingBioEntityFound) {
          if (isResultDrawerOpen) {
            dispatch(closeDrawer());
          }

          dispatch(clearBioEntitiesData());
          return;
        }
      }

      dispatch(selectTab(`${id}`));
      dispatch(openBioEntityDrawerById(id));


      PluginsEventBus.dispatchEvent('onSearch', {
        type: 'bioEntity',
        searchValues: [closestSearchResult],
        results: [bioEntityContents],
      });

      if (hasFitBounds) {
        searchFitBounds();
      }
    };
