import { PIN_ICON_ANY } from '@/constants/features';
import { Map } from 'ol';
import MapBrowserEvent from 'ol/MapBrowserEvent';

const isTargetHTMLElement = (target: string | HTMLElement | undefined): target is HTMLElement =>
  !!target && typeof target !== 'string' && 'style' in target;

/* prettier-ignore */
export const onPointerMove =
    (mapInstance: Map, event: MapBrowserEvent<PointerEvent>): void => {
      if (event.dragging) {
        return;
      }

      const pixel = mapInstance.getEventPixel(event.originalEvent);
      const feature = mapInstance.forEachFeatureAtPixel(pixel, firstFeature => {
        const isPinIcon = PIN_ICON_ANY.includes(firstFeature.get('type'));
        if (!isPinIcon) {
          return undefined;
        }

        return firstFeature;
      });

      const target = mapInstance.getTarget();
      if (isTargetHTMLElement(target)) {
        target.style.cursor = feature ? 'pointer' : '';
      }
    };
