import { getBioEntitiesIdsFromReaction } from '@/components/Map/MapViewer/utils/listeners/mapSingleClick/getBioEntitiesIdsFromReaction';
import { FIRST_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { openReactionDrawerById, selectTab } from '@/redux/drawer/drawer.slice';
import { openMapAndOrSetActiveIfSelected } from '@/redux/map/map.slice';
import { modelsNameMapSelector } from '@/redux/models/models.selectors';
import { getReactionsByIds } from '@/redux/reactions/reactions.thunks';
import type { AppDispatch, store } from '@/redux/store';
import type { BioEntityContent, Reaction } from '@/types/models';

interface Args {
  bioEntityContents: BioEntityContent[];
  dispatch: AppDispatch;
  getState: typeof store.getState;
}

const getReactionsIdsFromBioEntities = (bioEntites: BioEntityContent[]): number[] => {
  return bioEntites
    .filter(c => c?.bioEntity?.idReaction)
    .map(c => c?.bioEntity?.id)
    .filter((id): id is number => typeof id === 'number');
};

const fetchReactions = async (
  reactionsIds: number[],
  dispatch: AppDispatch,
): Promise<Reaction[]> => {
  const result = await dispatch(
    getReactionsByIds({
      ids: reactionsIds,
      shouldConcat: true,
    }),
  );

  // if has error (toast show should be handled by getReactionsByIds)
  if (typeof result.payload === 'string') {
    return [];
  }

  const reactions = result.payload?.data;
  if (!reactions) {
    return [];
  }

  return reactions;
};

const handleReactionShowInfoAndOpenMap = async (
  { dispatch, getState }: Args,
  firstReaction: Reaction,
): Promise<void> => {
  const modelsNames = modelsNameMapSelector(getState());

  dispatch(openReactionDrawerById(firstReaction.id));
  dispatch(selectTab(''));
  dispatch(
    openMapAndOrSetActiveIfSelected({
      modelId: firstReaction.modelId,
      modelName: modelsNames[firstReaction.modelId],
    }),
  );
};

export const fetchReactionsAndGetBioEntitiesIds = async (args: Args): Promise<string[]> => {
  const { dispatch, bioEntityContents } = args;

  const bioEntityReactionsIds = getReactionsIdsFromBioEntities(bioEntityContents || []);
  if (bioEntityReactionsIds.length === SIZE_OF_EMPTY_ARRAY) {
    return [];
  }

  const reactions = await fetchReactions(bioEntityReactionsIds, dispatch);
  if (reactions.length === SIZE_OF_EMPTY_ARRAY) {
    return [];
  }

  const bioEntitiesIds = reactions.map(reaction => getBioEntitiesIdsFromReaction(reaction)).flat();
  const firstReaction = reactions[FIRST_ARRAY_ELEMENT];
  if (firstReaction) {
    handleReactionShowInfoAndOpenMap(args, firstReaction);
  }

  return bioEntitiesIds;
};
