import { createSlice } from '@reduxjs/toolkit';
import { DRAWER_INITIAL_STATE } from './drawer.constants';
import {
  closeDrawerReducer,
  displayAddOverlaysDrawerReducer,
  displayBioEntitiesListReducer,
  displayChemicalsListReducer,
  displayDrugsListReducer,
  displayEntityDetailsReducer,
  displayGroupedSearchResultsReducer,
  getBioEntityChemicalsForTargetReducers,
  getBioEntityDrugsForTargetReducers,
  openBioEntityDrawerByIdReducer,
  openDrawerReducer,
  openOverlaysDrawerReducer,
  openReactionDrawerByIdReducer,
  openSearchDrawerWithSelectedTabReducer,
  openSubmapsDrawerReducer,
  selectTabReducer,
} from './drawer.reducers';

const drawerSlice = createSlice({
  name: 'drawer',
  initialState: DRAWER_INITIAL_STATE,
  reducers: {
    openDrawer: openDrawerReducer,
    openSearchDrawerWithSelectedTab: openSearchDrawerWithSelectedTabReducer,
    openSubmapsDrawer: openSubmapsDrawerReducer,
    openOverlaysDrawer: openOverlaysDrawerReducer,
    displayAddOverlaysDrawer: displayAddOverlaysDrawerReducer,
    selectTab: selectTabReducer,
    closeDrawer: closeDrawerReducer,
    displayDrugsList: displayDrugsListReducer,
    displayChemicalsList: displayChemicalsListReducer,
    displayBioEntitiesList: displayBioEntitiesListReducer,
    displayGroupedSearchResults: displayGroupedSearchResultsReducer,
    displayEntityDetails: displayEntityDetailsReducer,
    openReactionDrawerById: openReactionDrawerByIdReducer,
    openBioEntityDrawerById: openBioEntityDrawerByIdReducer,
  },
  extraReducers: builder => {
    getBioEntityDrugsForTargetReducers(builder);
    getBioEntityChemicalsForTargetReducers(builder);
  },
});

export const {
  openDrawer,
  openSearchDrawerWithSelectedTab,
  openSubmapsDrawer,
  openOverlaysDrawer,
  displayAddOverlaysDrawer,
  selectTab,
  closeDrawer,
  displayDrugsList,
  displayChemicalsList,
  displayBioEntitiesList,
  displayGroupedSearchResults,
  displayEntityDetails,
  openReactionDrawerById,
  openBioEntityDrawerById,
} = drawerSlice.actions;

export default drawerSlice.reducer;
