import type { DrawerName } from '@/types/drawerName';
import { KeyedFetchDataState } from '@/types/fetchDataState';
import { BioEntityContent, Chemical, Drug } from '@/types/models';
import { PayloadAction } from '@reduxjs/toolkit';

export type SearchDrawerState = {
  currentStep: number;
  stepType: 'bioEntity' | 'drugs' | 'chemicals' | 'none';
  selectedValue: BioEntityContent | Drug | Chemical | undefined;
  listOfBioEnitites: BioEntityContent[];
  selectedSearchElement: string;
};

export type OverlayDrawerState = {
  currentStep: number;
};

export type ReactionDrawerState = {
  reactionId?: number;
};

export type BioEntityDrawerState = {
  bioentityId?: number | string;
  drugs: KeyedFetchDataState<Drug[], []>;
  chemicals: KeyedFetchDataState<Chemical[], []>;
};

export type DrawerState = {
  isOpen: boolean;
  drawerName: DrawerName;
  searchDrawerState: SearchDrawerState;
  reactionDrawerState: ReactionDrawerState;
  bioEntityDrawerState: BioEntityDrawerState;
  overlayDrawerState: OverlayDrawerState;
};

export type OpenSearchDrawerWithSelectedTabReducerPayload = string;
export type OpenSearchDrawerWithSelectedTabReducerAction =
  PayloadAction<OpenSearchDrawerWithSelectedTabReducerPayload>;

export type OpenReactionDrawerByIdPayload = number;
export type OpenReactionDrawerByIdAction = PayloadAction<OpenReactionDrawerByIdPayload>;

export type OpenBioEntityDrawerByIdPayload = number | string;
export type OpenBioEntityDrawerByIdAction = PayloadAction<OpenBioEntityDrawerByIdPayload>;

export type SetSelectedSearchElementPayload = string;
export type SetSelectedSearchElementAction = PayloadAction<SetSelectedSearchElementPayload>;
