import type { AppStartListening } from '@/redux/store';
import { UNKNOWN_ERROR } from '@/utils/getErrorMessage/getErrorMessage.constants';
import { showToast } from '@/utils/showToast';
import {
  Action,
  createListenerMiddleware,
  isRejected,
  isRejectedWithValue,
} from '@reduxjs/toolkit';

export const errorListenerMiddleware = createListenerMiddleware();

const startListening = errorListenerMiddleware.startListening as AppStartListening;

export const errorMiddlewareListener = async (action: Action): Promise<void> => {
  if (isRejectedWithValue(action)) {
    let message: string;
    if (typeof action.payload === 'string') {
      message = action.payload;
    } else {
      message = UNKNOWN_ERROR;
    }

    showToast({
      type: 'error',
      message,
    });
  }
};

startListening({
  matcher: isRejected,
  effect: errorMiddlewareListener,
});
