import { mapModelSchema } from '@/models/modelSchema';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { MapModel } from '@/types/models';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { ThunkConfig } from '@/types/store';
import { MODELS_FETCHING_ERROR_PREFIX } from './models.constants';

export const getModels = createAsyncThunk<MapModel[] | undefined, void, ThunkConfig>(
  'project/getModels',
  async (_, { rejectWithValue }) => {
    try {
      const response = await axiosInstance.get<MapModel[]>(apiPath.getModelsString());

      const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapModelSchema));

      return isDataValid ? response.data : undefined;
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: MODELS_FETCHING_ERROR_PREFIX });

      return rejectWithValue(errorMessage);
    }
  },
);
