/* eslint-disable no-magic-numbers */
import { MAP_INITIAL_STATE } from '@/redux/map/map.constants';
import { RootState, store } from '@/redux/store';
import { bioEntityContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { DRAWER_INITIAL_STATE } from '@/redux/drawer/drawer.constants';

import { MODELS_DATA_MOCK_WITH_MAIN_MAP } from '@/redux/models/models.mock';
import { getVisibleBioEntitiesPolygonCoordinates } from './getVisibleBioEntitiesPolygonCoordinates';

jest.mock('../../../../redux/store');

const getStateSpy = jest.spyOn(store, 'getState');

describe('getVisibleBioEntitiesPolygonCoordinates', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });
  it('should return undefined if received array does not contain bioEntities with current map id', () => {
    getStateSpy.mockImplementation(
      () =>
        ({
          models: MODELS_DATA_MOCK_WITH_MAIN_MAP,
          map: {
            ...MAP_INITIAL_STATE,
            data: {
              ...MAP_INITIAL_STATE.data,
              modelId: 5052,
              size: {
                width: 256,
                height: 256,
                tileSize: 256,
                minZoom: 1,
                maxZoom: 1,
              },
            },
          },
          bioEntity: {
            loading: 'succeeded',
            error: { message: '', name: '' },
          },
          drugs: {
            loading: 'succeeded',
            error: { message: '', name: '' },
          },
          drawer: {
            ...DRAWER_INITIAL_STATE,
            bioEntityDrawerState: {
              bioentityId: undefined,
              drugs: {},
              chemicals: {},
            },
          },
        }) as RootState,
    );

    expect(getVisibleBioEntitiesPolygonCoordinates()).toBe(undefined);
  });
  it('should return coordinates, max zoom, and map instance if received array contain bioEntities with current map id and max zoom', () => {
    getStateSpy.mockImplementation(
      () =>
        ({
          models: MODELS_DATA_MOCK_WITH_MAIN_MAP,
          map: {
            ...MAP_INITIAL_STATE,
            data: {
              ...MAP_INITIAL_STATE.data,
              modelId: 52,
              size: {
                width: 256,
                height: 256,
                tileSize: 256,
                minZoom: 1,
                maxZoom: 5,
              },
            },
          },
          bioEntity: {
            data: [
              {
                searchQueryElement: bioEntityContentFixture.bioEntity.name,
                loading: 'succeeded',
                error: { name: '', message: '' },
                data: [
                  {
                    ...bioEntityContentFixture,
                    bioEntity: {
                      ...bioEntityContentFixture.bioEntity,
                      model: 52,
                      x: 97,
                      y: 53,
                      z: 1,
                    },
                  },
                  {
                    ...bioEntityContentFixture,
                    bioEntity: {
                      ...bioEntityContentFixture.bioEntity,
                      model: 52,
                      x: 12,
                      y: 25,
                      z: 1,
                    },
                  },
                ],
              },
            ],
            loading: 'succeeded',
            error: { message: '', name: '' },
          },
          drugs: {
            data: [
              {
                searchQueryElement: '',
                loading: 'succeeded',
                error: { name: '', message: '' },
                data: undefined,
              },
            ],
            loading: 'succeeded',
            error: { message: '', name: '' },
          },
          chemicals: {
            data: [
              {
                searchQueryElement: '',
                loading: 'succeeded',
                error: { name: '', message: '' },
                data: undefined,
              },
            ],
            loading: 'succeeded',
            error: { message: '', name: '' },
          },
          drawer: {
            ...DRAWER_INITIAL_STATE,
            bioEntityDrawerState: {
              bioentityId: undefined,
              drugs: {},
              chemicals: {},
            },
            searchDrawerState: {
              ...DRAWER_INITIAL_STATE.searchDrawerState,
              selectedSearchElement: bioEntityContentFixture.bioEntity.name,
            },
          },
        }) as RootState,
    );

    expect(getVisibleBioEntitiesPolygonCoordinates()).toEqual({
      mapInstance: null,
      maxZoom: 5,
      polygonCoordinates: [
        [-18158992, 11740728],
        [-4852834, 16123932],
      ],
    });
  });
});
